@@ -1,7 +1,7 @@
 package Rope;
 
 use 5.006; use strict; use warnings;
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
 use Rope::Object;
 my (%META, %PRO);
@@ -87,8 +87,7 @@ BEGIN {
 					my ($prop, $options) = (shift @properties, shift @properties);
 
 					my $ref = ref $options;					
-				
-					if (!$ref || $ref ne 'HASH' || ! grep { $options->{$_} } qw/initable writeable enumerable configurable value/) {
+					if (!$ref || $ref ne 'HASH' || ! grep { defined $options->{$_} } qw/initable writeable builder enumerable configurable value/) {
 						$options = {
 							initable => 1,
 							enumerable => 1,
@@ -238,10 +237,17 @@ BEGIN {
 							enumerable => 1,
 							configurable => 1,
 							index => ++$META{$caller}{keys}
-						};;
+						};
+					}
+				}
+				for ( sort { $build->{properties}->{$a}->{index} <=> $build->{properties}->{$b}->{index} } keys %{ $build->{properties} } ) {
+					if ( !defined $build->{properties}->{$_}->{value} && defined $build->{properties}->{$_}->{builder}) {
+						my $builder = $build->{properties}->{$_}->{builder};
+						$build->{properties}->{$_}->{value} = ref $builder ? $builder->($build) : $caller->$builder($build);
 					}
 				}
 				tie %{${$self}->{prototype}}, 'Rope::Object', $PRO{scope}($self, %{$build});
+
 				return $self;
 			};
 		}
@@ -261,6 +267,8 @@ sub import {
 			keys => 0
 		};
 	}
+
+	$PRO{keyword}($caller, '(bool', sub { scalar keys %{${$_[0]}->{prototype}}; });
 	$PRO{keyword}($caller, '((', sub {});
 	$PRO{keyword}($caller, '(%{}', sub {
 		${$_[0]}->{prototype};
@@ -300,19 +308,7 @@ sub new {
 	$PRO{extends}($name)(@{$meta->{extends}}) if ($meta->{extends});
 	$PRO{with}($name)(@{$meta->{with}}) if ($meta->{with});
 	$PRO{properties}($name)(ref $meta->{properties} eq 'ARRAY' ? @{$meta->{properties}} : %{$meta->{properties}}) if ($meta->{properties});
-
-	my $self = \{
-		prototype => {}
-	};
-
-	bless $self, $name;
-
-	tie %{${$self}->{prototype}}, 'Rope::Object', $PRO{scope}($self, %{$PRO{clone}($META{$name})});
-	for (keys %params) {
-		$self->{$_} = $params{$_};
-	}
-
-	return $self;
+	return $PRO{new}($name)($name, %params);
 }
 
 1;
@@ -325,17 +321,12 @@ Rope - Tied objects
 
 =head1 VERSION
 
-Version 0.08
+Version 0.09
 
 =cut
 
 =head1 SYNOPSIS
 
-Quick summary of what the module does.
-
-Perhaps a little code snippet.
-
-
 	package Knot;
 
 	use Rope;
@@ -347,7 +338,7 @@ Perhaps a little code snippet.
 
 	);
 
-	properties {
+	properties (
 		bends => {
 			type => sub { $_[0] =~ m/^\d+$/ ? $_[0] : die "$_[0] != integer" },
 			value => 10,
@@ -357,7 +348,7 @@ Perhaps a little code snippet.
 			required => 1
 		},
 		...
-	};
+	);
 
 	function add_loops => sub {
 		my ($self, $loop) = @_;
@@ -378,11 +369,138 @@ Perhaps a little code snippet.
 
 	$k->{add_loops} = 5; # errors
 
-... and/or ... 
+
+=head1 DESCRIPTION
+
+C<Rope> is an Object Orientation system that is built on top of perls core hash tying implementation. It extends the functionality which is available with all the modern features you would expect from an modern OO system. This includes clear class and role definitions. With rope you also get out of the box sorted objects, where the order you define you class persists.
+
+=head1 CONFIGURE PROPERTIES
+
+=head2 initable
+
+If set to a true value then this property can be initialised during the object creationg, when calling ->new(%params). If set to false then on initialisation ion the code will die with a relevant error. (Cannot initalise Object ($name) property ($key) as initable is not set to true.)
+
+=head2 writeable
+
+If set to a true value then this property value can be updated after initialisation with any other value. If set to false then the code will die with a relevant error. (Cannot set Object ($name) property ($key) it is only readable)
+
+=head2 configurable
+
+If set to a true value then this property value can be updated after initialisation with a value that matches the type of the existing. If you try to set a value which is not of the same type the error will die with a relevant error. (Cannot change Object ($name) property ($key) type). If you set to false and writeable is also false you will get the same error as writeable false.
+
+=head2 enumerable
+
+If set to a true value then the property will be enumerable when you itterate the object for example when you call keys %{$self}. If set to false then the property will be hidden from any iteration. Note also that when itterating your object keys are already ordered based on the order they were assigned.
+
+=head2 required
+
+If set to a true value then this property is required at initialisation, either by a value key being set or via passing into ->new. I would suggest using this in conjunction with initable when you require a value is passed. If no value is passed an required is set to true then the code will die with a relevant error. (Required property ($key) in object ($object) not set)
+
+=head2 type
+
+The type property/key expects a code ref to passed, all values that are then set either during initialisation or writing from with the following code will run through this ref. Rope expects that you return the final value from this sub routine so you can use coercion via a type.
+
+=head2 builder
+
+The buidler property/key expects either a code ref or a scalar that represents the name of a sub routine in your class (currently not another function/property but may extend in the future). It expects the value for that property to be returned from either the code ref or sub routine. Within a builder you can also add new properties to your object by exstend the passed defintion, when extending this way i would suggest using ++$_[0]->{keys} to set the index so that sorting is persistent further down the line.
+
+=head2 index
+
+The index property/key expects an integer, if you do not set then this integer is automatically generated and associated to the property. You will only want to set this is you always want to have a property last when itterating
+
+=head1 KEYWORDS
+
+=head2 property
+
+Extends the current object definition with a single new property
+
+	property one => (
+		initable => 1,
+		writeable => 0,
+		enumerable => 1,
+		builder => sub {
+			return 200;
+		}
+	);
+
+
+=head2 properties
+
+Extends the current object definition with multiple new properties
+
+	properties (
+		two => {
+			type => sub { $_[0] =~ m/^\d+$/ ? $_[0] : die "$_[0] != integer" },
+			value => 10,
+			initable => 1,
+			configurable => 1,
+			enumerable => 1,
+			required => 1
+		},
+		...
+	);
+
+=head2 prototyped
+
+Extends the current object definition with multiple new properties where initable, writable and enumerable are all set to a true value.
+
+	prototyped (
+		two => 10
+		...
+	);
+
+=head2 function
+
+Extends the current object definition with a new property that acts as a function. A function has initable, writeable, enumerable and configurable all set to false so it cannot be changed/set once the object is instantiated.
+
+	function three => sub {
+		my ($self, $param) = @_;
+		...
+	};
+	
+=head2 extends
+
+The extends keyword allows you to extend your current definition with another object, your objext will inherit all the properties of that extended object.
+
+	package Ping;
+	use Rope;
+	extends 'Pong';
+
+=head2 with
+
+The with keyword allows you to include roles in your current object definition, your objext will inherit all the properties of that role.
+
+	package Ping;
+	use Rope;
+	with 'Pong';
+
+=head2 requires
+
+The requires keyword allows you to define properties which are required for either a role or an object, it works in both directions.
+
+	package Pong;
+	use Rope::Role;
+	requires qw/host/;
+	function ping => sub { ... };
+	function pong => sub { ... };
+
+	package Ping;
+	use Rope;
+	requires qw/ping pong/;
+	with 'Pong';
+	prototyped (
+		host => '...'
+	);
+
+=head1 METHODS
+
+=head2 new
+
+Along with class definitions you can also generate object using Rope itself, the options are the same as described above.
 
 	my $knot = Rope->new({
 		name => 'Knot',
-		properties => {
+		properties => [
 			loops => 1,
 			hitches => {
 				type => Int,
@@ -394,7 +512,7 @@ Perhaps a little code snippet.
 				my ($self, $loop) = @_;
 				$self->{loops} += $loop;
 			}
-		}
+		]
 	});
 
 	my $with = Rope->new({
@@ -407,6 +525,89 @@ Perhaps a little code snippet.
 	$knot->{loops};
 	$with->loops;
 
+=head1 CLASS DEFINITION
+
+	package Builder;
+
+	use Rope;
+
+	property one => (
+		initable => 1,
+		writeable => 0,
+		enumerable => 1,
+		builder => sub {
+			return 200;
+		}	
+	);
+
+	property two => (
+		writeable => 0,
+		enumerable => 0,
+		builder => sub {
+			$_[0]->{properties}->{three} = {
+				value => 'works',
+				writeable => 0,
+				index => ++$_[0]->{keys}
+			};
+			return $_[0]->{properties}->{one}->{value} + 11;
+		}
+	);
+
+	1;
+
+
+=head1 ROLE DEFINITION
+
+	package Builder::Role;
+
+	use Rope::Role;
+
+	property one => (
+		initable => 1,
+		writeable => 0,
+		enumerable => 1,
+		builder => sub {
+			return 200;
+		}	
+	);
+
+	1;
+
+=head1 AUTOLOADING
+
+If you do not enjoy accessing properties as hash keys and would instead like to access them as package routines then you can simply include C<Rope::Autoload> and this will use perls internal AUTOLOAD functionality to expose your properties as routines.
+
+	package Builder;
+
+	use Rope;
+	use Rope::Autoload;
+
+	...
+
+	1;
+
+So you can write
+
+	$builder->thing = 10;
+
+Instead of
+
+	$builder->{thing} = 10;
+
+=head1 TYPES
+
+Rope also includes additional helpers for defining properties with fixed types, see C<Rope::Type> for more information. ( internally that uses C<Type::Standard> for the actual type checking. )
+
+	package Knot;
+	 
+	use Rope;
+	use Rope::Type qw/int/;
+	 
+	int loops => 1;
+	int hitches => 10;
+
+	1;
+
 =head1 AUTHOR
 
 LNATION, C<< <email at lnation.org> >>
